ltc = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1","AceConsole-2.0")
local Gratuity = AceLibrary("Gratuity-2.0")
local Compost = AceLibrary("Compost-2.0")

local colours = {
	[0] = "|cff88aaff",
	[1] = "|cffffff33",
	[2] = "|cffffff33",
	[3] = "|cffff8033",
	[4] = "|cffff0033",
}

local Rcolour = "|cff00ff33"

local Plugins = Compost:GetTable ()
local CostList = Compost:GetTable ()
local ReagentList = Compost:GetTable ()
local ReagentCount = Compost:GetTable ()
local CountList = Compost:GetTable ()

local L = AceLibrary("AceLocale-2.2"):new("lern2count")

L:RegisterTranslations("enUS", function()
	return {
		["(%d+) Mana"] = true,
		["(%d+) Rage"] = true,
		["(%d+) Focus"] = true,
		["(%d+) Energy"] = true,
		["(%d+) Health"] = true,
	}
end)

L:RegisterTranslations("frFR", function()
	return {
		["(%d+) Mana"] = "Mana : (%d+)",
		["(%d+) Rage"] = "Rage : (%d+)",
		["(%d+) Focus"] = "Focalisation : (%d+)",
		["(%d+) Energy"] = "Energie : (%d+)",
		["(%d+) Health"] = "(%d+) points de vie",
	}
end)


local CostTable = {
	{ type = 0, reg = L["(%d+) Mana"] },
	{ type = 1, reg = L["(%d+) Rage"] },
	{ type = 2, reg = L["(%d+) Focus"] },
	{ type = 3, reg = L["(%d+) Energy"] },
	{ type = 4, reg = L["(%d+) Health"] },
}

function ltc:OnEnable()
	self:RegisterEvent("UNIT_MANA","PowerUpdate")
	self:RegisterEvent("UNIT_RAGE","PowerUpdate")
	self:RegisterEvent("UNIT_ENERGY","PowerUpdate")
	if not AceLibrary:HasInstance("SpecialEvents-Bags-2.0") then
		self:RegisterEvent("BAG_UPDATE")
	else
		self:RegisterEvent("SpecialEvents_BagSlotUpdate")
	end

	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED","BarUpdate")
	self:RegisterEvent("ACTIONBAR_HIDEGRID","BarUpdate")
	self:RegisterEvent("ACTIONBAR_PAGE_CHANGED","BarUpdate")
	self:RegisterEvent("ACTIONBAR_UPDATE_USABLE","BarUpdate")
	self:RegisterEvent("ACTIONBAR_PAGE_CHANGED","BarUpdate")

    self:ScheduleEvent(self.DelayedUpdate, 1, self) --Delay initializing the addon a second, thats enough for everything to finish what its doing (I hope)

	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	self:Hook("ActionButton_UpdateCount")

end

function ltc:PLAYER_REGEN_DISABLED()
	self.combat = true
	self:PowerUpdate ("player")
end

function ltc:PLAYER_REGEN_ENABLED()
	self.combat = false
	self:PowerUpdate ("player")
end

local GratuityHandler = {
	action = function (id) Gratuity:SetAction (id) end,
	pet = function (id) Gratuity:SetPetAction (id) end,
	shapeshift = function (id) Gratuity:SetShapeshift (id) end,
}

function ltc:UpdateCount(stringo, info)
	local ptype = UnitPowerType("player")

	GratuityHandler[info.type](info.id)
	
	local ctype, cost, colour, amount
	if CostList[stringo] then
		ctype = CostList[stringo].type
		cost = CostList[stringo].cost
	end
	
	colour = colours[ctype]
	if info.type == "pet" then
		amount = UnitMana("pet")
	elseif ctype == 4 then
		amount = UnitHealth("player")
	else
		amount = UnitMana("player")
	end
	
	local rcount
	do
		local reagent = ReagentList[stringo]
		if reagent then
			rcount = ReagentCount[reagent]
			if not rcount then
				rcount = self:GetInvCount (reagent)
				ReagentCount[reagent] = rcount
			end
		end
	end
	local pcount

	if cost then
		if ctype == 3 and ptype == 3 then -- Energy
			pcount = ceil( (cost - amount) / 20 )
			if pcount <= 0 then pcount = nil end
		else
			pcount = floor(amount / cost)
		end
		if pcount == 0 and ctype == 1 then -- Rage
			pcount = nil
		end
	end

	local value
	if self.combat then
		if rcount then
			if pcount and pcount > rcount then
				value = colour..pcount.."|r"
			else
				value = Rcolour..rcount.."|r"
			end
		elseif pcount then
			value = colour..pcount.."|r"
		end
	else
		if rcount then
			value = Rcolour..rcount.."|r"
		elseif pcount then
			value = colour..pcount.."|r"
		end
	end
		
	if not value and info.type == "action" and IsConsumableAction (info.id) then
		value = GetActionCount (info.id)
	end

	getglobal(stringo):SetText (value or "")
end

function ltc:ActionButton_UpdateCount()
	local name = this:GetName().."Count"
	local info = CountList[name]
	if info then
		self:UpdateCount (name, info)
	else
		self.hooks.ActionButton_UpdateCount ()
	end
end

function ltc:DelayedUpdate()

	self:CountListUpdate()
	self:CostAndReagentListUpdate()
	self:PowerUpdate("player")

end

function ltc:RegisterPlugin(func)
	table.insert(Plugins, func)
end

function ltc:CountListUpdate()
	Compost:Reclaim(CountList, 2)
	CountList = Compost:GetTable ()

	for _, func in ipairs(Plugins) do
	 	func (CountList)
	end
end

function ltc:CostAndReagentListUpdate()
	Compost:Reclaim(CostList, 2)
	CostList = Compost:GetTable ()
	
	ReagentList = Compost:Erase (ReagentList)
	ReagentCount = Compost:Erase (ReagentCount)
	
	for stringo, info in pairs(CountList) do
		GratuityHandler[info.type](info.id)
		
		for _,c in pairs(CostTable) do
			local _,_,cost = Gratuity:Find(c.reg, 2, 2, false, true)

			if cost then
				cost = tonumber(cost)
				CostList[stringo] = Compost:AcquireHash ('type', c.type, 'cost', cost)
				break
			end
		end
		
		local count
		local _,_,reagent = Gratuity:Find(SPELL_REAGENTS.."(.+)", 2, 99, false, true)
		if reagent then
			if string.sub(reagent, 0, 2) == "|c" then
				reagent = string.sub(reagent, 11, -3)
			end
			ReagentList[stringo] = reagent
		end
	end
end

local function GetItemName(link)
	local _, _, id = string.find(link, "item:(%d+):%d+:%d+:%d+")
	if id then
		id = tonumber (id)
		if id then
			local name = GetItemInfo(id)
			return name
		end
	end
end

function ltc:SpecialEvents_BagSlotUpdate(bag, slot, itemlink, stack, oldlink, oldstack)
	if oldlink then
		local reagent = GetItemName(oldlink)
		if reagent then
			local count = ReagentCount[reagent]
			if count then
				ReagentCount[reagent] = count - oldstack
			end
		end
	end
	if itemlink then
		local reagent = GetItemName(itemlink)
		if reagent then
			count = ReagentCount[reagent]
			if count then
				ReagentCount[reagent] = count + stack
			end
		end
	end
end

function ltc:BAG_UPDATE()
	ReagentCount = Compost:Erase (ReagentCount)
end


function ltc:PowerUpdate(unit)
	if unit ~= "player" then
		return
	end
	

	for stringo, info in pairs(CountList) do
		self:UpdateCount (stringo, info)
	end
end

function ltc:GetInvCount(item)
	local val = 0
	for b = 0, NUM_BAG_FRAMES do
     	for s = 1, GetContainerNumSlots(b) do
			local c = GetContainerItemLink(b, s)
			if c then
				local cName = GetItemName (c)
				if cName == item then
					local _, amount = GetContainerItemInfo(b, s)
					val = val + amount
				end
			end
		end
	end
	return val
end

function ltc:BarUpdate(slot)
	self:ScheduleEvent(self.DelayedUpdate, 0.1, self)
end
